﻿//
// Copyright (C) 2011, ZAO Nanosoft. All rights reserved.
// 
namespace HostMgdReversi
{
  using System;
  using System.Collections.Generic;
  using System.Linq;
  using System.Text;

  #if NCAD
    using HostMgd.ApplicationServices;
    using HostMgd.EditorInput;
    
    using Teigha.DatabaseServices;
    using Teigha.Runtime;
    using Teigha.Geometry;
    
    using Platform = Teigha;
    using Host = HostMgd;
  #else
    using Autodesk.AutoCAD.DatabaseServices;
    using Autodesk.AutoCAD.Runtime;
    using Autodesk.AutoCAD.Geometry;
    using Autodesk.AutoCAD.ApplicationServices;
    using Autodesk.AutoCAD.EditorInput;
    
    using Platform = Autodesk.AutoCAD;
    using Host = Autodesk.AutoCAD;
  #endif

  class Desk
  {
    Database db = null;

    public System.Collections.Generic.Dictionary<ObjectId, Piece> GameDesc = new System.Collections.Generic.Dictionary<ObjectId, Piece>();
    public Piece[,] GameDesc_xy = new Piece[8, 8];

    public void Init()
    {
      this.db = Application.DocumentManager.MdiActiveDocument.Database;
    }

    /// <summary>
    /// Add horizontal border of the Game Board
    /// </summary>
    /// <param name="row">row number</param>
    private void DrawRowLine(int row, BlockTableRecord ms, Transaction tr)
    {
      PolygonMesh mesh = new PolygonMesh();
      mesh.NSize = 2;
      mesh.MSize = 2;
      ms.AppendEntity(mesh);
      tr.AddNewlyCreatedDBObject(mesh, true);
      AddVertexToMesh(mesh, new Point3d(0, row * Constants.gridstep, -Constants.linehight), tr);
      AddVertexToMesh(mesh, new Point3d(0, row * Constants.gridstep,  Constants.linehight), tr);
      AddVertexToMesh(mesh, new Point3d(8 * Constants.gridstep, row * Constants.gridstep, -Constants.linehight), tr);
      AddVertexToMesh(mesh, new Point3d(8 * Constants.gridstep, row * Constants.gridstep,  Constants.linehight), tr);
    }

    /// <summary>
    /// Add vertical border of the Game Board
    /// </summary>
    /// <param name="col">Column number</param>
    private void DrawColumnLine(int col, BlockTableRecord ms, Transaction tr)
    {
      PolygonMesh mesh = new PolygonMesh();
      mesh.NSize = 2;
      mesh.MSize = 2;
      ms.AppendEntity(mesh);
      tr.AddNewlyCreatedDBObject(mesh, true);
      AddVertexToMesh(mesh, new Point3d(col * Constants.gridstep, 0, -Constants.linehight), tr);
      AddVertexToMesh(mesh, new Point3d(col * Constants.gridstep, 0,  Constants.linehight), tr);
      AddVertexToMesh(mesh, new Point3d(col * Constants.gridstep, 8 * Constants.gridstep, -Constants.linehight), tr);
      AddVertexToMesh(mesh, new Point3d(col * Constants.gridstep, 8 * Constants.gridstep,  Constants.linehight), tr);
    }

    /// <summary>
    /// Create Game Board cell
    /// </summary>
    /// <param name="row">Row number</param>
    /// <param name="col">Column number</param>
    public void CreatePiece(int col, int row, ePlayer player)
    {
      var gamecell = GameDesc_xy[col, row];
      gamecell.DrawPiece();
      gamecell.Player = player;
    }
    private void CreatePiece(int col, int row, ePlayer player, BlockTableRecord ms, Transaction tr)
    {
      var gamecell = GameDesc_xy[col, row];
      gamecell.DrawPiece(ms, tr);
      gamecell.Player = player;
    }

    /// <summary>
    /// Create Game Board
    /// </summary>
    public void DrawDesk()
    {
      var tm = db.TransactionManager;

      using (Transaction myT = tm.StartTransaction())
      {
        BlockTable bt = (BlockTable)myT.GetObject(db.BlockTableId, OpenMode.ForRead, false);
        BlockTableRecord btr = (BlockTableRecord)myT.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForWrite, false);

        // Create borders between cells
        for (int idx = 0; idx < 9; ++idx)
        {
          DrawRowLine(idx, btr, myT);
          DrawColumnLine(idx, btr, myT);
        }

        /// Create Game Board
        /// 8 x 8 cells
        for (int i = 0; i < 8; ++i)
        {
          for (int j = 0; j < 8; ++j)
          {
            Piece cell = new Piece(i, j);
            ObjectId objId = cell.DrawCell(btr, myT);
            this.GameDesc.Add(objId, cell);
            this.GameDesc_xy[i, j] = cell;
          }
        }

        // Create initial disks position
        CreatePiece(3, 4, ePlayer.Human, btr, myT);
        CreatePiece(4, 3, ePlayer.Human, btr, myT);
        CreatePiece(3, 3, ePlayer.PC, btr, myT);
        CreatePiece(4, 4, ePlayer.PC, btr, myT);

        myT.Commit();
      }
    }
    #region Private methods
    private void AddVertexToMesh(PolygonMesh PolyMesh, Point3d Pt3d, Transaction Trans)
    {
      PolygonMeshVertex PMeshVer = new PolygonMeshVertex(Pt3d);
      PolyMesh.AppendVertex(PMeshVer);
      Trans.AddNewlyCreatedDBObject(PMeshVer, true);
    }
    #endregion
  }
}
