﻿//
// Copyright (C) 2011, ZAO Nanosoft. All rights reserved.
//
namespace HostMgdReversi
{
  using System;
  using System.Collections.Generic;
  using System.Linq;
  using System.Text;

#if NCAD
  using HostMgd.ApplicationServices;
  using HostMgd.EditorInput;

  using Teigha.DatabaseServices;
  using Teigha.Runtime;
  using Teigha.Geometry;

  using Platform = Teigha;
  using Host = HostMgd;
#else
    using Autodesk.AutoCAD.DatabaseServices;
    using Autodesk.AutoCAD.Runtime;
    using Autodesk.AutoCAD.Geometry;
    using Autodesk.AutoCAD.ApplicationServices;
    using Autodesk.AutoCAD.EditorInput;
    
    using Platform = Autodesk.AutoCAD;
    using Host = Autodesk.AutoCAD;
#endif

  class ScorePanel
  {
    Database db = null;

    ObjectId white_scoreId = ObjectId.Null;
    ObjectId black_scoreId = ObjectId.Null;
    ObjectId curr_actionId = ObjectId.Null;
    ObjectId leader_Id     = ObjectId.Null;

    public void Init()
    {
      db = Application.DocumentManager.MdiActiveDocument.Database;
    }

    public void DrawPanel()
    {
      var tm = db.TransactionManager;

      using (Transaction myT = tm.StartTransaction())
      {
        BlockTable bt = (BlockTable)myT.GetObject(db.BlockTableId, OpenMode.ForRead, false);
        BlockTableRecord btr = (BlockTableRecord)myT.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForWrite, false);

        DrawText(new Point3d(-256, 360, 0), "Reversi .Net", btr, myT);
        DrawText(new Point3d(-245, 275, 0), "Disks", btr, myT);
        DrawText(new Point3d(-235, 235, 0), "blue:", btr, myT);
        DrawText(new Point3d(-235, 195, 0), "black:", btr, myT);

        this.white_scoreId = DrawText(new Point3d(-135, 235, 0), "2", btr, myT);
        this.black_scoreId = DrawText(new Point3d(-135, 195, 0), "2", btr, myT);
        this.curr_actionId = DrawText(new Point3d(-256, 315, 0), "Move: blue", btr, myT);
        this.leader_Id = DrawText(new Point3d(-256, 80, 0), "Score: draw", btr, myT);

        DrawRect(-295, 0, -45, 400, btr);
        DrawRect(-265, 170, -90, 340, btr);

        myT.Commit();
      }
    }

    public string WhiteScore
    {
      set
      {
        ChangeText(this.white_scoreId, value);
      }
      get
      {
        return GetText(this.white_scoreId);
      }
    }

    public string BlackScore
    {
      set
      {
        ChangeText(this.black_scoreId, value);
      }
      get
      {
        return GetText(this.black_scoreId);
      }
    }

    public string CurrentAction
    {
      set
      {
        ChangeText(this.curr_actionId, value);
      }
      get
      {
        return GetText(this.curr_actionId);
      }
    }

    public string Leader
    {
      set
      {
        ChangeText(this.leader_Id, value);
      }
      get
      {
        return GetText(this.leader_Id);
      }
    }

    private void ChangeText(ObjectId labelId, string text)
    {
      var tm = db.TransactionManager;

      using (Transaction myT = tm.StartTransaction())
      {
        BlockTable bt = myT.GetObject(db.BlockTableId, OpenMode.ForRead, false) as BlockTable;
        BlockTableRecord btr = myT.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForWrite, false) as BlockTableRecord;
        DBText label = myT.GetObject(labelId, OpenMode.ForWrite, false) as DBText;

        label.TextString = text;

        myT.Commit();
      }
    }

    private string GetText(ObjectId labelId)
    {
      var tm = db.TransactionManager;

      using (Transaction myT = tm.StartTransaction())
      {
        BlockTable bt = myT.GetObject(db.BlockTableId, OpenMode.ForRead, false) as BlockTable;
        BlockTableRecord btr = myT.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForRead, false) as BlockTableRecord;
        DBText label = myT.GetObject(labelId, OpenMode.ForRead, false) as DBText;

        return label.TextString;
      }
    }

    private ObjectId DrawText(Point3d insertPoint, string text, BlockTableRecord btr, Transaction tr)
    {
      TextStyleTable tst = (TextStyleTable)tr.GetObject(db.TextStyleTableId, OpenMode.ForRead, false);

      var dtext = new DBText();
      dtext.SetDatabaseDefaults();

      dtext.TextString = text;
      dtext.Height = 15;
      dtext.Position = insertPoint;

      ObjectId tid = btr.AppendEntity(dtext);
      tr.AddNewlyCreatedDBObject(dtext, true);

      return tid;
    }

    private ObjectId DrawRect(int x1, int y1, int x2, int y2, BlockTableRecord ms)
    {
      var tm = db.TransactionManager;

      Polyline pln = new Polyline();
      pln.SetDatabaseDefaults();
      pln.ColorIndex = 9;

      Point2d pt1 = new Point2d(x1, y1);
      Point2d pt2 = new Point2d(x2, y1);
      Point2d pt3 = new Point2d(x2, y2);
      Point2d pt4 = new Point2d(x1, y2);

      pln.AddVertexAt(0, pt3, 0, 0, 0);
      pln.AddVertexAt(1, pt4, 0, 0, 0);
      pln.AddVertexAt(2, pt1, 0, 0, 0);
      pln.AddVertexAt(3, pt2, 0, 0, 0);
      pln.Closed = true;

      ObjectId pid = ms.AppendEntity(pln);
      tm.AddNewlyCreatedDBObject(pln, true);

      return pid;
    }
  }
}
